/*
 *
 * Copyright (c) Nextian. All rights reserved.
 *
 * This software is furnished under a license. Use, duplication,
 * disclosure and all other uses are restricted to the rights
 * specified in the written license agreement.
 */
package com.nextian.icmpping.helpers;

import com.nextian.icmpping.helpers.handlers.PingRequest;
import com.nextian.icmpping.jni.IcmpEchoResponse;

/**
 * Interface containing asynchronous callbacks for {@link Ping}.
 */
public interface PingResponseHandler {
    /**
     * Callback method for a successful ICMP echo response.
     *
     * @param request  request passed to {@link Ping#echo echo()} method for which handler is called
     * @param response response message received
     */
    void handleResponse(PingRequest request, IcmpEchoResponse response);

    /**
     * Callback method for timeout (timeout value is passed as an argument to {@link Ping#echo echo()}).
     *
     * @param request request passed to {@link Ping#echo echo()} method for which handler is called
     */
    void handleTimeout(PingRequest request);

    /**
     * Callback method for exceptions. An exceptions is typically related to underlying raw socket, and it is
     * very likely that the parent {@link Ping} instance (calling the handler) is no longer usable
     * (the best strategy is to close the parent Ping object).
     *
     * @param request   request passed to {@link Ping#echo echo()} method for which handler is called
     * @param exception underlying exceptions information
     */
    void handleException(PingRequest request, Exception exception);
}
