/*
 *
 * Copyright (c) Nextian. All rights reserved.
 *
 * This software is furnished under a license. Use, duplication,
 * disclosure and all other uses are restricted to the rights
 * specified in the written license agreement.
 */
package com.nextian.icmpping.helpers.examples;

import com.nextian.icmpping.helpers.Ping;
import com.nextian.icmpping.helpers.PingResult;
import com.nextian.icmpping.helpers.exceptions.PingErrorException;

import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;

/**
 * Synchronous (blocking) command line helpers.
 */
public class PingExampleSync {
    /**
     * Convert list of DNS host names into IP addresses. Invalid (non-resolvable) addresses are skipped.
     *
     * @param args DNS host names
     * @return resolved IP host addresses
     */
    private static InetAddress[] getAddresses(String[] args) {
        InetAddress address;
        InetAddress[] addresses;
        ArrayList<InetAddress> list = new ArrayList<InetAddress>();

        for (int i = 1; i < args.length; i++) {
            try {
                address = Inet4Address.getByName(args[i]);
                if (address != null) {
                    list.add(address);
                } else {
                    System.out.println("Unknown host: " + args[i]);
                }
            } catch (UnknownHostException ex) {
                System.out.println("Unknown host: " + args[i]);
            }
        }
        addresses = list.toArray(new InetAddress[list.size()]);
        return addresses;
    }

    /**
     * Ping IP addresses sequentially.
     *
     * @param addresses list of hosts addresses to helpers
     * @param timeout   timeout (milliseconds)
     */
    private static void sequentialPing(InetAddress[] addresses, int timeout) {
        PingResult[] results;

        try {
            results = Ping.echo(addresses, "PING DATA".getBytes(), timeout);
            for (PingResult result : results) {
                switch (result.getResult()) {
                    case RESPONSE:
                        System.out.println(result.getAddress().getHostAddress() + " response: " + result.getStatus() + " time:" + result.getResponseTime());
                        break;
                    case TIMEOUT:
                        System.out.println(result.getAddress().getHostAddress() + " timeout");
                        break;
                    case ERROR:
                        System.out.println(result.getAddress().getHostAddress() + "helpers error");
                        break;
                }
            }
        } catch (PingErrorException ex) {
            System.err.println("Fatal error - terminating " + ex.toString());
        } catch (InterruptedException ex) {
            System.err.println("Ping interrupted - terminating");
        }
    }

    public static void main(String[] args) {
        if (args.length >= 2) {
            int timeout = Integer.parseInt(args[0]);
            InetAddress[] addresses = getAddresses(args);

            try {
                Ping.init();
                sequentialPing(addresses, timeout);
            } catch (PingErrorException ex) {
                System.err.println("Could not initialize helpers: " + ex.toString());
            } finally {
                Ping.release();
            }
        } else {
            System.out.println("Usage:");
            System.out.println("     Ping <timeout> <address list>");
            System.out.println("");
        }
    }
}
