/*
 *
 * Copyright (c) Nextian. All rights reserved.
 *
 * This software is furnished under a license. Use, duplication,
 * disclosure and all other uses are restricted to the rights
 * specified in the written license agreement.
 */
package com.nextian.icmpping.jni.exception;

import com.nextian.icmpping.jni.IcmpSocket;

/**
 * The exceptions is thrown by calls to {@link IcmpSocket}, when underlying WinSock calls fail.
 */
public class IcmpSystemException extends IcmpException {
    /**
     * Underlying WinSock error code.
     */
    private final int errorCode;

    /**
     * Exception constructor.
     *
     * @param errorCode          Winsock error code
     * @param nativeErrorMessage Winsock error message describing the error code
     */
    public IcmpSystemException(int errorCode, String nativeErrorMessage) {
        super(nativeErrorMessage);
        this.errorCode = errorCode;
    }

    /**
     * Accessor to {@link #errorCode}.
     *
     * @return {@link #errorCode}
     */
    public final int getErrorCode() {
        return errorCode;
    }
}
