/*
 * Copyright (c) Nextian. All rights reserved.
 *
 * This software is furnished under a license. Use, duplication,
 * disclosure and all other uses are restricted to the rights
 * specified in the written license agreement.
 *
 */
package com.nextian.icmpping.helpers;

import com.nextian.icmpping.jni.IcmpSocket;
import com.nextian.icmpping.jni.exception.IcmpSystemException;
import org.apache.log4j.Logger;

import java.util.concurrent.atomic.AtomicInteger;

/**
 * ICMP request sender.
 */
public class Sender implements Runnable {
    private final Logger logger;
    private final int requestsCount;
    private final AtomicInteger sentCounter;
    private final IcmpSocket icmp;
    private final TestData testData;

    /**
     * Construct sender and initialize private members
     *
     * @param logger         error logger
     * @param icmp           icmp socket
     * @param requestsToSend number of requests to be sent
     * @param sentCounter    count of actual requests sent
     */
    public Sender(Logger logger, IcmpSocket icmp, int requestsToSend, AtomicInteger sentCounter) {
        this.logger = logger;
        this.icmp = icmp;
        this.requestsCount = requestsToSend;
        this.sentCounter = sentCounter;
        testData = new TestData();
    }

    public void run() {
        final int sendInterval = 5; // send every ICMP request every 5ms
        try {
            for (int i = 0; i < requestsCount; i++) {
                icmp.send(testData.getNextHost(), icmp.getNextMessageId(), TestData.DEFAULT_ICMP_TEST_PACKET_DATA, TestData.DEFAULT_ICMP_TEST_PACKET_DATA.length);
                Thread.sleep(sendInterval);
                sentCounter.incrementAndGet();
            }
        } catch (IcmpSystemException e) {
            logger.error(e.getMessage());
        } catch (InterruptedException e) {
            logger.error("Interrupted");
        }
    }
}
