/*
Copyright (c) Nextian. All rights reserved.

This software is furnished under a license. Use, duplication,
disclosure and all other uses are restricted to the rights
specified in the written license agreement.
*/

#include <stdio.h>
#include <tchar.h>
#include <winsock.h>
#include "Icmp.h"
#include "IcmpDebug.h"

static UINT16 InternetChecksum(u_short *addr, int len)
{
	register int nleft = len;
	register u_short *w = addr;
	register u_short answer;
	register int sum = 0;

	while (nleft > 1) {
		sum += *w++;
		nleft -= 2;
	}

	if (nleft == 1) {
		u_short	u = 0;

		*(u_char *)(&u) = *(u_char *)w;
		sum += u;
	}

	sum = (sum >> 16) + (sum & 0xffff);
	sum += (sum >> 16);
	answer = (u_short)(~sum);
	return answer;
}

int IcmpInitialize()
{
	WORD wVersionRequested;
	WSADATA wsaData;
	int	iResult;

	ICMPOUTPUTDEBUGSTRING0(_T(__FUNCTION__) _T(": IN\n"));
	wVersionRequested = MAKEWORD(2, 2);
	iResult = WSAStartup(wVersionRequested, &wsaData);
	if (iResult != 0 /* failure */)
	{
		ICMPOUTPUTDEBUGSTRING1(_T(__FUNCTION__) _T(": WSA startup failed, result=%d\n"), iResult);
	}

	ICMPOUTPUTDEBUGSTRING1(_T(__FUNCTION__) _T(": OUT, result=%d\n"), iResult);
	return iResult;
}

void IcmpRelease()
{
	int iResult;

	ICMPOUTPUTDEBUGSTRING0(_T(__FUNCTION__) _T(": IN\n"));
	iResult = WSACleanup();
	if (iResult == SOCKET_ERROR /* failure */)
	{
		int iError;
		iError = WSAGetLastError();
		ICMPOUTPUTDEBUGSTRING1(_T(__FUNCTION__) _T(": WSACleanup() failed, WSAGetLastError() returned %d\n"), iError);
	}

	ICMPOUTPUTDEBUGSTRING0(_T(__FUNCTION__) _T(": OUT\n"));
}

SOCKET IcmpCreateSocket()
{
	SOCKET s;

	ICMPOUTPUTDEBUGSTRING0(_T(__FUNCTION__) _T(": IN\n"));
	s = socket(AF_INET, SOCK_RAW, IPPROTO_ICMP);
	if (s == INVALID_SOCKET /* failure */)
	{
		ICMPOUTPUTDEBUGSTRING1(_T(__FUNCTION__) _T(" socket() failed (no administrative privileges?), WSAGetLastError() returned %d\n"), WSAGetLastError());
	}

	ICMPOUTPUTDEBUGSTRING1(_T(__FUNCTION__) _T(": OUT, returned socket handle=%u\n"), s);
	return s;
}

int IcmpCloseSocket(SOCKET socket)
{
	int iResult;
	u_long iMode = 1;

	ICMPOUTPUTDEBUGSTRING1(_T(__FUNCTION__) _T(": IN, socket=%u\n"), socket);
	iResult = ioctlsocket(socket, FIONBIO, &iMode);
	if (iResult == NO_ERROR)
	{
		iResult = closesocket(socket);
		if (iResult == SOCKET_ERROR)
		{
			ICMPOUTPUTDEBUGSTRING2(_T(__FUNCTION__) _T(": closesocket() failed for socket %u, error %d\n"), socket, WSAGetLastError());
		}
	}
	else
	{
		ICMPOUTPUTDEBUGSTRING2(_T(__FUNCTION__) _T(": ioctlsocket() failed on socket %u, error %d\n"), socket, WSAGetLastError());
	}

	ICMPOUTPUTDEBUGSTRING2(_T(__FUNCTION__) _T(": OUT, socket=%u, result=%d\n"), socket, iResult);
	return iResult;
}

int IcmpWait(SOCKET rawSocket, long timeout)
{
	struct timeval timeoutVal;
	fd_set readFd;
	int result;

	ICMPOUTPUTDEBUGSTRING2(_T("IcmpWait IN: Socket %u timeout %ld"), rawSocket, timeout);
	if (timeout >= 0)
	{
		timeoutVal.tv_sec = timeout / 1000;
		timeoutVal.tv_usec = 1000 * (timeout % 1000);

		readFd.fd_count = 1;
		readFd.fd_array[0] = rawSocket;

		result = select(1, &readFd, NULL, NULL, &timeoutVal);
	}
	else result = 0;
	if (result != SOCKET_ERROR)
		ICMPOUTPUTDEBUGSTRING2(_T("IcmpWait OUT: Socket %u select OK result=%d"), rawSocket, result);
	else
		ICMPOUTPUTDEBUGSTRING2(_T("IcmpWait OUT: Socket %u select failed %d"), rawSocket, WSAGetLastError());
	return result;
}

int IcmpGetSocketError(SOCKET socket)
{
	int iError;
	int iErrorCodeLen = sizeof(iError);
	int iResult;

	iResult = getsockopt(socket, SOL_SOCKET, SO_ERROR, (char *)&iError, &iErrorCodeLen);
	if (iResult == SOCKET_ERROR)
	{
		ICMPOUTPUTDEBUGSTRING2(_T(__FUNCTION__) _T(": getsockopt() failed on socket %u, error %d\n"), socket, iResult);
		iError = WSAEINVAL;
	}
	return iError;
}

int IcmpGetLastError()
{
	int iError = WSAGetLastError();
	ICMPOUTPUTDEBUGSTRING1(_T("IcmpGetLastError: Get last error %d\n"), iError);
	return iError;
}

int IcmpSendEcho(SOCKET rawSocket, const INT8 *destRawAddress, UINT16 identifier, UINT16 sequenceNumber, const void *dataBuffer, long dataLength)
{
	IcmpMessage echoRequest;
	struct sockaddr_in dstAddr;
	long requestLength;
	int result;

	ICMPOUTPUTDEBUGSTRING1(_T("IcmpSendEcho IN: Socket %u\n"), rawSocket);
	memset(&dstAddr, 0, sizeof(dstAddr));
	if (dataLength <= ICMP_ECHO_MAX_DATA_LEN && !IsBadReadPtr(destRawAddress, sizeof(dstAddr.sin_addr.S_un.S_addr)) && !IsBadReadPtr(dataBuffer, dataLength))
	{
		/* Copy raw IP address (4 bytes) to sockaddr structure */
		dstAddr.sin_family = AF_INET;
		memcpy(&dstAddr.sin_addr.S_un.S_addr, destRawAddress, sizeof(dstAddr.sin_addr.S_un.S_addr));
		ICMPOUTPUTDEBUGSTRING5(_T("IcmpSendEcho: Socket %u IP=%u.%u.%u.%u"), rawSocket, (UINT8)(destRawAddress[0]), (UINT8)(destRawAddress[1]), (UINT8)(destRawAddress[2]), (UINT8)(destRawAddress[3]));

		/* Build ICMP Echo request */
		memset(&echoRequest, 0, sizeof(echoRequest));
		echoRequest.header.type = ICMP_ECHOREQEST;                  // RFC 792 type: 8 = ECHO_REQUEST
		echoRequest.header.code = 0;                                // RFC 792 type: code
		echoRequest.header.quench.echo.id = htons(identifier);      // RFC 792 type: identification
		echoRequest.header.quench.echo.seq = htons(sequenceNumber); // RFC 792 type: sequenceNumber
		memcpy(echoRequest.data.buffer, dataBuffer, dataLength);
		requestLength = sizeof(IcmpMessage) - ICMP_ECHO_MAX_DATA_LEN + dataLength;
		echoRequest.header.checksum = InternetChecksum((u_short *)&echoRequest, (int)requestLength);

		result = sendto(rawSocket, (const char*)&echoRequest, (int)requestLength, 0, (struct sockaddr *)&dstAddr, sizeof(struct sockaddr_in));
	}
	else
	{
		WSASetLastError(WSAEINVAL);
		result = SOCKET_ERROR;
	}
	if (result != SOCKET_ERROR)
		ICMPOUTPUTDEBUGSTRING2(_T("IcmpSendEcho OUT: Socket %u sendto() OK"), rawSocket, dstAddr.sin_addr.S_un.S_addr);
	else
		ICMPOUTPUTDEBUGSTRING2(_T("IcmpSendEcho OUT: Socket %u sendto() failed %d"), rawSocket, WSAGetLastError());
	return result;
}

int IcmpReadMessage(SOCKET socket, IcmpResponseMessage *response)
{
	struct sockaddr_in sourceAddress;
	int sourceAddLen = sizeof(struct sockaddr_in);
	int result;

	ICMPOUTPUTDEBUGSTRING1(_T("IcmpReadMessage IN: Socket %u"), socket);
	if (!IsBadReadPtr(response, sizeof(IcmpResponseMessage)))
	{
		result = recvfrom(socket, (char *)response, sizeof(IcmpResponseMessage), 0, (struct sockaddr *)&sourceAddress, &sourceAddLen);
	}
	else
	{
		WSASetLastError(WSAEINVAL);
		result = SOCKET_ERROR;
	}

	if (result != SOCKET_ERROR)
	{
		ICMPOUTPUTDEBUGSTRING1(_T("IcmpReadMessage OUT: Socket %u recvfrom() OK"), socket);
	}
	else
	{
		ICMPOUTPUTDEBUGSTRING2(_T("IcmpReadMessage OUT: Socket %u recvrom() failed %d"), socket, WSAGetLastError());
	}
	return result;
}

BOOL IcmpDecodeResponseMessage(const IcmpResponseMessage *message, UINT8 *srcAddress, UINT8 *dstAddress, UINT8 *type, UINT8 *code, UINT16 *identifier, UINT16 *sequenceNumber, void *dataBuffer, size_t dataBuffferLength, long *dataLength)
{
	u_short len;
	BOOL result;

	ICMPOUTPUTDEBUGSTRING0(_T("IcmpDecodeResponseMessage IN:\n"));
	if (!IsBadReadPtr(message, sizeof(IcmpResponseMessage)) && !IsBadWritePtr(srcAddress, 4 * sizeof(UINT8)) && !IsBadWritePtr(type, sizeof(INT8)) && !IsBadWritePtr(code, sizeof(INT8)) && !IsBadWritePtr(identifier, sizeof(UINT16)) && !IsBadWritePtr(sequenceNumber, sizeof(UINT16)) && !IsBadWritePtr(dataLength, sizeof(long)))
	{
		ICMPOUTPUTDEBUGSTRING1(_T("IcmpDecodeResponseMessage: Decode IP protocol %d\n"), message->ipHeader.protocol);
		if (message->ipHeader.protocol == IP_PROTOCOL_ICMP)
		{
			ICMPOUTPUTDEBUGSTRING1(_T("ICMP: Decode message type %d\n"), message->icmp.header.type);
			switch (message->icmp.header.type)
			{
			case ICMP_ECHOREPLY:
				len = ntohs(message->ipHeader.length) - ICMP_PAYLOAD_MIN_LEN - IP_HEADER_LEN;
				if (len <= dataBuffferLength && !IsBadWritePtr(dataBuffer, len))
				{
					*type = message->icmp.header.type;
					*code = message->icmp.header.code;
					*identifier = ntohs(message->icmp.header.quench.echo.id);
					*sequenceNumber = ntohs(message->icmp.header.quench.echo.seq);
					memcpy(dataBuffer, message->icmp.data.buffer, len);
					*dataLength = len;
					memcpy(srcAddress, &(message->ipHeader.iaddrSrc.S_un.S_addr), sizeof(message->ipHeader.iaddrSrc.S_un.S_addr));
					/* There is no echo destination address in response but we assume the echo response is sent from echo request destination */
					memcpy(dstAddress, &(message->ipHeader.iaddrSrc.S_un.S_addr), sizeof(message->ipHeader.iaddrSrc.S_un.S_addr));
					result = TRUE;
				}
				else
				{
					WSASetLastError(WSAEFAULT);
					result = FALSE;
				}
				break;
			case ICMP_DESTINATIONUNREACHABLE:
			case ICMP_SOURCEQUENCH:
			case ICMP_REDIRECT:
			case ICMP_ECHOREQEST:
			case ICMP_TIMEEXCEEDED:
			case ICMP_PARAMETERPROBLEM:
				*type = message->icmp.header.type;
				*code = message->icmp.header.code;
				*identifier = ntohs(message->icmp.data.packet.icmp.quench.echo.id);
				*sequenceNumber = ntohs(message->icmp.data.packet.icmp.quench.echo.seq);
				*dataLength = 0;
				memcpy(srcAddress, &(message->ipHeader.iaddrSrc.S_un.S_addr), sizeof(message->ipHeader.iaddrSrc.S_un.S_addr));
				memcpy(dstAddress, &(message->icmp.data.packet.ipHeader.iaddrDst.S_un.S_addr), sizeof(message->ipHeader.iaddrSrc.S_un.S_addr));
				result = TRUE;
				break;
			default:
				WSASetLastError(WSAEPROTONOSUPPORT);
				result = FALSE;
				break;
			}
		}
		else
		{
			WSASetLastError(WSAEPROTONOSUPPORT);
			result = FALSE;
		}
	}
	else
	{
		WSASetLastError(WSAEINVAL);
		result = FALSE;
	}
	if (result == TRUE)
		ICMPOUTPUTDEBUGSTRING0(_T("IcmpDecodeResponseMessage OUT: Decode message OK\n"));
	else
		ICMPOUTPUTDEBUGSTRING1(_T("IcmpDecodeResponseMessage OUT: Decode message failed %d\n"), result);
	return result;
}

LPTSTR IcmpGetErrorString(int errorCode)
{
	LPTSTR pMessage;
	DWORD dwResult;

	dwResult = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_ARGUMENT_ARRAY | FORMAT_MESSAGE_IGNORE_INSERTS, NULL, errorCode, 0, (LPTSTR)&pMessage, 0, NULL);
	if (dwResult == 0 /* error */)
	{
		ICMPOUTPUTDEBUGSTRING0(_T(__FUNCTION__) _T(": FormatMessage() failed"));
		pMessage = NULL;
	}
	return pMessage;
}

void IcmpReleaseErrorString(LPTSTR errorMessage)
{
	if (errorMessage != NULL)
	{
		LocalFree(errorMessage);
	}
	else
	{
		ICMPOUTPUTDEBUGSTRING0(_T(__FUNCTION__) _T(": NULL passed as argument\n"));
	}
}
