/*
Copyright (c) Nextian. All rights reserved.

This software is furnished under a license. Use, duplication,
disclosure and all other uses are restricted to the rights
specified in the written license agreement.
*/

#ifndef __ICMPDEBUG_H__
#define __ICMPDEBUG_H__

#include <wtypes.h>

/// <summary>Maximum character output of IcmpOutputDebugString()</summary>
#define ICMP_DEBUG_OUTPUT_BUFFER_SIZE 4096

#ifdef __cplusplus
extern "C" {
#endif

	/// <summary>Print formatted message to Windows debug output via OutputDebugString (maximum ICMP_DEBUG_OUTPUT_BUFFER_SIZE-1 characters are printed)</summary>
	/// <param name="format">format of message to be printed (as in vsprintf)</param>
	/// <returns>Number of characters written or -1 on failure</returns>
	int IcmpOutputDebugString(const TCHAR *format, ...);

	// Macros for output debug messages (expand to nothing in release mode, returning 
	// 0 as no characters written)
#ifdef _DEBUG
#define ICMPOUTPUTDEBUGSTRING0(f)		IcmpOutputDebugString(f)
#define ICMPOUTPUTDEBUGSTRING1(f,a1)	IcmpOutputDebugString(f,a1)
#define ICMPOUTPUTDEBUGSTRING2(f,a1,a2)	IcmpOutputDebugString(f,a1,a2)
#define	ICMPOUTPUTDEBUGSTRING5(f, a1, a2, a3, a4, a5) IcmpOutputDebugString(f, a1, a2, a3, a4, a5)
#else
#define ICMPOUTPUTDEBUGSTRING0(f)		0
#define ICMPOUTPUTDEBUGSTRING1(f,a1)	0
#define ICMPOUTPUTDEBUGSTRING2(f,a1,a2)	0
#define ICMPOUTPUTDEBUGSTRING5(f,a1,a2,a3,a4,a5) 0

#endif

#ifdef __cplusplus
}
#endif

#endif
