/*
Copyright (c) Nextian. All rights reserved.

This software is furnished under a license. Use, duplication,
disclosure and all other uses are restricted to the rights
specified in the written license agreement.
*/

#include <wtypes.h>
#include <winbase.h>
#include <tchar.h>
#include "Sddl.h"
#include "IcmpDebug.h"
#include "IcmpMutex.h"

HANDLE IcmpCreateMutex()
{
	static const TCHAR * const ICMP_MUTEX_NAME = _T("NEXTIAN_ICMP_MUTEX");
	static const TCHAR * const ICMP_MUTEX_SECURITY_DESCRIPTOR = _T("D:(A;NP;0x001f0001;;;WD)");

	HANDLE handle;				// resulting mutex handle
	SECURITY_ATTRIBUTES sa;		// security attributes structure
	BOOL result;				// operation result
	ULONG dummy = 0;			// dummy 

	ICMPOUTPUTDEBUGSTRING0(_T("IcmpCreateMutex: function entry\n"));

	handle = OpenMutex(SYNCHRONIZE, FALSE, ICMP_MUTEX_NAME);
	if (handle == NULL /* mutex does not exist, create */)
	{
		ICMPOUTPUTDEBUGSTRING0(_T("IcmpCreateMutex: mutex not found, creating\n"));

		// Initialize security attributes
		memset(&sa, 0, sizeof(sa));
		sa.nLength = sizeof(SECURITY_ATTRIBUTES);
		sa.bInheritHandle = 0;
		sa.lpSecurityDescriptor = 0;

		// Convert security descriptor string (ICMP_MUTEX_SECURITY_DESCRIPTOR)) to security descriptor
		result = ConvertStringSecurityDescriptorToSecurityDescriptor(ICMP_MUTEX_SECURITY_DESCRIPTOR, SDDL_REVISION_1, &(sa.lpSecurityDescriptor), &dummy);
		if (result != 0 /* success */)
		{
			handle = CreateMutex(&sa, FALSE, ICMP_MUTEX_NAME);
			LocalFree(sa.lpSecurityDescriptor);
		}
		else
		{
			ICMPOUTPUTDEBUGSTRING0(_T("IcmpCreateMutex: failure in ConvertStringSecurityDescriptorToSecurityDescriptor()\n"));
		}
	}
	else
	{
		ICMPOUTPUTDEBUGSTRING0(_T("IcmpCreateMutex: mutex already created, returning existing mutex\n"));
	}
	ICMPOUTPUTDEBUGSTRING1(_T("IcmpCreateMutex: leaving function, returned mutex handle=%p\n"), handle);
	return handle;
}

BOOL IcmpCloseMutex(HANDLE hMutex)
{
	BOOL result;

	ICMPOUTPUTDEBUGSTRING1(_T("IcmpCloseMutex: entering function, hMutes=%p\n"), hMutex);
	result = CloseHandle(hMutex);
	ICMPOUTPUTDEBUGSTRING1(_T("IcmpCloseMutex: leaving function, result=%d\n"), result);
	return result;
}

BOOL IcmpLockMutex(HANDLE hMutex)
{
	BOOL result;

	ICMPOUTPUTDEBUGSTRING1(_T("IcmpLockMutex: function entry, hMutex=%p\n"), hMutex);
	result = (BOOL)(WaitForSingleObject(hMutex, INFINITE) == WAIT_OBJECT_0);
	ICMPOUTPUTDEBUGSTRING1(_T("IcmpLockMutex: leaving function, result=%d\n"), result);
	return result;
}

BOOL IcmpUnlockMutex(HANDLE hMutex)
{
	BOOL result;

	ICMPOUTPUTDEBUGSTRING1(_T("IcmpUnlockMutex: function entry, hMutex=%p\n"), hMutex);
	result = ReleaseMutex(hMutex);
	ICMPOUTPUTDEBUGSTRING1(_T("IcmpUnlockMutex: leaving function, result=%d\n"), result);
	return result;
}
