/*
Copyright (c) Nextian. All rights reserved.

This software is furnished under a license. Use, duplication,
disclosure and all other uses are restricted to the rights
specified in the written license agreement.
*/

#ifndef __ICMP_MUTEX_H__
#define __ICMP_MUTEX_H__

#include <wtypes.h>

#ifdef __cplusplus
extern "C" {
#endif

	/// <summary>Creates and opens ICMP mutex. Call IcmpCloseMutex() when done.</summary>
	/// <returns>mutex handle</returns>
	/// <seealso cref="IcmpCloseMutex()"/>
	HANDLE IcmpCreateMutex();

	/// <summary>Close mutex created by IcmpCreateMutex()</summary>
	/// <param name='hMutex'>mutex handle</param>
	/// <returns>TRUE on success or FALSE on failure</returns>
	BOOL IcmpCloseMutex(HANDLE hMutex);

	/// <summary>Obtain a lock on a mutex created by IcmpCreateMutex()</summary>
	/// <param name='hMutex'>mutex handle</param>
	/// <returns>TRUE on success or FALSE on failure</returns>
	/// <seealso cref="IcmpUnlockMutex()"/>
	BOOL IcmpLockMutex(HANDLE hMutex);

	/// <summary>Unlock/release a lock on a mutex locked by IcmpLockMutex()</summary>
	/// <param name='hMutex'>handle of mutex to unlock</param>
	/// <returns>TRUE on success or FALSE on failure</returns>
	/// <seealso cref="IcmpLockMutex()"/>
	BOOL IcmpUnlockMutex(HANDLE hMutex);

#ifdef __cplusplus
}
#endif

#endif
