/*
Copyright (c) Nextian. All rights reserved.

This software is furnished under a license. Use, duplication,
disclosure and all other uses are restricted to the rights
specified in the written license agreement.
*/

#include "CppUnitTest.h"
#include "IcmpMutex.h"

using namespace Microsoft::VisualStudio::CppUnitTestFramework;

namespace WindowsIcmpTest
{
	TEST_CLASS(MutexTest)
	{
	public:
		/// Test mutex open and close. Mutex creation should always succeed so later
		/// closing mutex returns TRUE
		TEST_METHOD(MutexOpenAndClose)
		{
			HANDLE hMutex;

			hMutex = IcmpCreateMutex();
			Assert::IsNotNull(hMutex);
			Assert::AreEqual(IcmpCloseMutex(hMutex), TRUE);
		}

		/// Test double open (mutex is a singleton, the same mutex should always be returned)
		TEST_METHOD(DoubleMutexOpenAndClose)
		{
			HANDLE hMutex1, hMutex2;

			hMutex1 = IcmpCreateMutex();
			Assert::IsNotNull(hMutex1);
			hMutex2 = IcmpCreateMutex();
			Assert::IsNotNull(hMutex2);

			Assert::AreNotSame(hMutex1, hMutex2);

			Assert::AreEqual(IcmpCloseMutex(hMutex1), TRUE);
			Assert::AreEqual(IcmpCloseMutex(hMutex2), TRUE);
		}

		/// Test locking and unlocking sequence
		TEST_METHOD(LockUnlock)
		{
			HANDLE hMutex = NULL;

			Assert::IsNull(hMutex);
			Assert::IsTrue(IcmpUnlockMutex(NULL) == 0);		// unlock NULL mutex
			hMutex = IcmpCreateMutex();
			Assert::IsNotNull(hMutex);
			Assert::IsTrue(IcmpUnlockMutex(hMutex) == 0);	// unlock not locked mutex
			Assert::IsTrue(IcmpLockMutex(hMutex) != 0);		// proper lock/unlock sequence
			Assert::IsTrue(IcmpUnlockMutex(hMutex) != 0);
			Assert::IsTrue(IcmpCloseMutex(hMutex) != 0);	// close mutex
			Assert::IsTrue(IcmpCloseMutex(hMutex) == 0);	// attempt to close already closed mutex
		}
	};
}