/*
Copyright (c) Nextian. All rights reserved.

This software is furnished under a license. Use, duplication,
disclosure and all other uses are restricted to the rights
specified in the written license agreement.
*/

#include "CppUnitTest.h"
#include "Icmp.h"
#include "IcmpDebug.h"

using namespace Microsoft::VisualStudio::CppUnitTestFramework;

namespace WindowsIcmpTest
{
	/// Socket call tests
	TEST_CLASS(SocketTest)
	{
	public:
		/// Initialize sockets for tests
		TEST_CLASS_INITIALIZE(StartupSockets)
		{
			WSADATA wsaData;
			int iResult = WSAStartup(MAKEWORD(2, 2), &wsaData);
			Assert::IsTrue(iResult == 0);
		}

		/// Shutdown sockets after tests
		TEST_CLASS_CLEANUP(ShutdownSockets)
		{
			int iResult = WSACleanup();
			Assert::IsTrue(iResult == 0);
		}

		/// Open and close socket (opening RAW sockets requires elevated privileges,
		/// make sure that you've run Visual Studio as administrator)
		TEST_METHOD(CreateAndCloseSocket)
		{
			SOCKET socket = IcmpCreateSocket();
			LPTSTR sError = IcmpGetErrorString(IcmpGetLastError());

			Assert::IsTrue(socket != SOCKET_ERROR);
			Assert::IsTrue(IcmpCloseSocket(socket) != SOCKET_ERROR);
			IcmpOutputDebugString(sError);
			IcmpReleaseErrorString(sError);
		}

		/// Close invalid socket 
		TEST_METHOD(CloseInvalidSocket)
		{
			Assert::IsTrue(IcmpCloseSocket((SOCKET)-1) == SOCKET_ERROR);
		}

		/// Get socket error (success)
		TEST_METHOD(GetSocketError)
		{
			SOCKET socket = IcmpCreateSocket();
			Assert::IsTrue(socket != SOCKET_ERROR);

			int result = IcmpGetSocketError(socket);
			Assert::IsTrue(result == 0 /* success */);

			Assert::IsTrue(IcmpCloseSocket(socket) == 0);
		}

		/// Call IcmpGetSocketError() on an invalid socket
		TEST_METHOD(GetSocketErrorInvalidSocket)
		{
			int result = IcmpGetSocketError((SOCKET)-1);

			Assert::IsTrue(result == WSAEINVAL);
		}

		/// Attempt to close an invalid socket and retrieve error message
		TEST_METHOD(GetErrorStringInvalidSocket)
		{
			LPTSTR sError;
			int iErr;

			Assert::IsTrue(IcmpCloseSocket(static_cast<SOCKET>(-1)) == SOCKET_ERROR);
			iErr = IcmpGetLastError();
			sError = IcmpGetErrorString(iErr);
			Assert::IsNotNull(sError);
			IcmpOutputDebugString(sError);
			IcmpReleaseErrorString(sError);
		}

		/// Releasing NULL string should not cause program to break, only 
		/// an warning sent via OutputDebugString (regardless of Debug or 
		/// Release mode of the library)
		TEST_METHOD(TestReleaseNullErrorString)
		{
			IcmpReleaseErrorString(NULL);
		}
	};
}
